#! /local/bin/tclsh7.6

#
# ----------------------------------------------------------------------
# Gemeinsame Funktionen von PPL-Check und der verschiedenen CGI -
# Skripte zum Verwalten der Fragen
# ----------------------------------------------------------------------
#

set HAVE_LOADQ 1
set loadqrev {$Id: loadq.tcl,v 1.4 1997/12/03 18:08:35 fp Exp $}

# ----------------------------------------------------------------------
# Lese Fragen ein
# ----------------------------------------------------------------------

proc wfn {line} {
    while {![eof $line]} {
	set input [string trim [gets $line]]
	if {[string length $input] > 0} {
	    break
	}
    }
    if [eof $line] {
	return ""
    }
    return $input
}

proc extract {tf kat i} {
    global frage

    if [regexp "\{(\[^\}]*)\}" $tf dummy m] {
	regsub " *\{(\[^\}]*)\} *" $tf {} tf
	foreach eq [split $m :] {
	    set q [split $eq =]
	    if {[llength $q] > 1} {
		eval set frage($kat,$i,[lindex $q 0]) \"[lindex $q 1]\"
	    } else {
		eval set frage($kat,$i,[lindex $q 0]) 1
	    }
	}
    }
    set frage($kat,$i) $tf
}

proc readq {kat line} {
    global frage antwort

    set title [gets $line]
    set rev   [gets $line]
    set index 0
    set i2    0

    set input [wfn $line]

    if [eof $line] {
	return 0
    }
    while {$input != "----"} {
	extract $input $kat $index
	if {$input == "-"} {
	    set input [wfn $line]
	    incr index
	    continue
	}
	catch {set frage($kat,$index,a) [string trim [gets $line]]}
	catch {set frage($kat,$index,b) [string trim [gets $line]]}
	catch {set frage($kat,$index,c) [string trim [gets $line]]}
	catch {set frage($kat,$index,d) [string trim [gets $line]]}

	if [eof $line] {
	    error "unerwartetes EOF in Frage [expr $index + 1]"
	}

	if {[string length $frage($kat,$index,a)] == 0 ||
	    [string length $frage($kat,$index,b)] == 0 ||
	    [string length $frage($kat,$index,c)] == 0 ||
	    [string length $frage($kat,$index,d)] == 0} {
	    error "weniger als vier Antworten fuer Frage [expr $index + 1]"
	}

	set input [string trim [gets $line]]
	if {[string length $input] > 0} {
	    error "mehr als vier Antworten fuer Frage [expr $index + 1]"
	}

	set input [wfn $line]

	if [eof $line] {
	    error "unerwartetes EOF in Frage [expr $index + 1]"
	}

	incr index
    }

    set input [wfn $line]

    while {![eof $line]} {
	foreach blubb [split [string trim $input]] {
	    set antwort($kat,$i2) $blubb
	    if {$frage($kat,$i2)=="-" && $antwort($kat,$i2)!="-"} {
		error "leere Frage [expr $i2 + 1], nicht-leere Antwort"
	    }
	    if {$frage($kat,$i2)!="-" && $antwort($kat,$i2)=="-"} {
		error "nicht-leere Frage [expr $i2 + 1], leere Antwort"
	    }
	    incr i2
	}
	set input [wfn $line]
    }

    if {$i2 != $index} {
	error "$index Fragen, $i2 Antworten"
    }

    set frage($kat,title) $title
    set frage($kat,rev)   $rev
    set frage($kat,count) $index

    return $index
}

proc loadq {kat file} {
    set f [open $file]
    set q [readq $kat $f]
    close $f
    return $q
}

# ----------------------------------------------------------------------
# Zufallsfunktion
# ----------------------------------------------------------------------

#
# Tcl 8.0 hat einen eingebauten Zufallsgenerator
#

if [catch {set blubber [expr rand()]}] {
    proc rand {max} {
	global _ran
	set _ran [expr ($_ran * 9301 + 49297) % 233280]
	return [expr int($max * ($_ran / double(233280)))]
    }
    proc srand {seed} {
	global _ran
	set _ran [expr $seed % 233280]
    }
    set _ran [expr (11*[pid] + [clock seconds]) % 233280]
} else {
    proc rand {max} {
	return [expr int($max * rand())]
    }
    proc srand {seed} {
	expr srand($seed)
    }
}

# ----------------------------------------------------------------------
# Permutations-Array aufsetzen
# ----------------------------------------------------------------------

if [catch {set perm(0,a)}] {
    set i 0
    foreach s {abcd abdc acbd acdb adbc adcb \
	    bacd badc bcad bcda bdac bdca \
	    cabd cadb cbad cbda cdab cdba \
	    dabc dacb dbac dbca dcab dcba} {
	set perm($i,a) [string index $s 0]
	set perm($i,b) [string index $s 1]
	set perm($i,c) [string index $s 2]
	set perm($i,d) [string index $s 3]
	incr i
    }
}

#
# ----------------------------------------------------------------------
# Ganz praktisch: Setze eine Variable, wenn sie noch nicht definiert ist
# ----------------------------------------------------------------------
#

proc ifndef { var val } {
    global $var
    if { [ catch { set $var } ] } {
        set $var $val
    }
}

#
# ----------------------------------------------------------------------
# Zeige eine Textdatei in einem Text-Widget
# ----------------------------------------------------------------------
#

proc DisplayFile {fname {title ""} {top ""}} {
    global tcl_platform _df_ctr

    ifndef _df_ctr 0
    incr _df_ctr

    if {$top == ""} {
	set top .file[set _df_ctr]
	toplevel $top
    } else {
	if [catch {toplevel $top}] {
	    return
	}
    }

    if [catch {set file [open $fname]} err] {
	destroy $top
	error $err
    }

    if {$title == ""} {
	set title [file rootname [file tail $fname]]
	set title "[string toupper [string index $title 0]][string range $title 1 end]"
    }

    wm title $top $title

    frame $top.top
    frame $top.bot
    frame $top.top.top
    frame $top.top.bot
    scrollbar $top.top.top.sby -command "$top.top.top.text yview"
    scrollbar $top.top.bot.sbx -command "$top.top.top.text xview" \
	    -orient horizontal
    text $top.top.top.text -relief sunken -bd 2 -wrap none \
	    -xscrollcommand "$top.top.bot.sbx set" \
	    -yscrollcommand "$top.top.top.sby set"
    frame $top.top.bot.dum \
	    -width [expr [lindex [$top.top.top.sby configure -width] 4] + \
	    2 * [lindex [$top.top.top.sby configure -borderwidth] 4] + \
	    [lindex [$top.top.top.text configure -borderwidth] 4]] \
	    -height [expr [lindex [$top.top.bot.sbx configure -width] 4] + \
	    2 * [lindex [$top.top.bot.sbx configure -borderwidth] 4]]

    if {$tcl_platform(platform) == "windows"} {
	$top.top.top.text configure -font systemfixed
    }

    pack $top.top.top.sby -side right -fill y
    pack $top.top.top.text -fill both -expand 1
    pack $top.top.bot.dum -side right
    pack $top.top.bot.sbx -side bottom -fill x
    pack $top.top.bot -side bottom -fill x
    pack $top.top.top -side top -fill both -expand 1

    button $top.bot.done -text "Schlieen" -command "destroy $top"
    pack $top.bot.done -side right -padx 4 -pady 4

    pack $top.bot -side bottom
    pack $top.top -side top -fill both -expand 1

    $top.top.top.text insert 1.0 [read $file]
    $top.top.top.text configure -state disabled
    close $file

    bind $top <Key-Prior>  "$top.top.top.text yview scroll -1 pages"
    bind $top <Key-Next>   "$top.top.top.text yview scroll  1 pages"
    bind $top <Key-Up>     "$top.top.top.text yview scroll -1 units"
    bind $top <Key-Down>   "$top.top.top.text yview scroll  1 units"
    bind $top <Key-Right>  "$top.top.top.text xview scroll  1 units"
    bind $top <Key-Left>   "$top.top.top.text xview scroll -1 units"
    bind $top <Key-Return> "destroy $top"
    focus $top.bot.done
}
