/*
 * Rand module for Tcl
 */

#include <tcl.h>
#include <stdlib.h>

static int
TclRand_Rand (ClientData clientData, Tcl_Interp *interp,
	      int argc, char *argv[])
{
  char thenum[256];

  if (argc != 1) {
    Tcl_SetResult (interp, "wrong # args: should be \"rand\"", TCL_STATIC);
    return TCL_ERROR;
  }

  sprintf (thenum, "%d", (int) rand());

  Tcl_SetResult (interp, thenum, TCL_VOLATILE);
  return TCL_OK;
}

static int
TclRand_Srand (ClientData clientData, Tcl_Interp *interp,
	       int argc, char *argv[])
{
  int theseed;

  if (argc != 2) {
    Tcl_SetResult (interp, "wrong # args: should be \"srand seed\"",
		   TCL_STATIC);
    return TCL_ERROR;
  }

  if (Tcl_GetInt (interp, argv[1], &theseed) != 0) {
    Tcl_SetResult (interp, "argument to \"srand\" must be integer",
		   TCL_STATIC);
    return TCL_ERROR;
  }

  srand ((unsigned int) theseed);

  return TCL_OK;
}

int
Random_Init (Tcl_Interp * interp)
{
  char thenum[256];

  sprintf (thenum, "%d", (int) RAND_MAX);

  Tcl_CreateCommand (interp, "rand",  TclRand_Rand,  NULL, NULL);
  Tcl_CreateCommand (interp, "srand", TclRand_Srand, NULL, NULL);
  Tcl_SetVar        (interp, "RAND_MAX", thenum, TCL_GLOBAL_ONLY);

  return TCL_OK;
}
